import pandas as pd

# An object once it has been created is called an instance
# The act of creating an object from a class constructor is called instantiation

# one instance of a dataframe object
df = pd.DataFrame({
    'a': [1,2,3],
    'b': [4,5,6]
})

# second instance of a dataframe object
df1 = pd.DataFrame({
    'a': [1,2,3],
    'b': [4,5,6]
})

# third instance of a dataframe object
df2 = pd.DataFrame({
    'a': [1,2,3],
    'b': [4,5,6]
})

# Class Variables are called "Attributes"
# Attributes do not use parentheses
print(df.shape) # -> (3, 2)
print(df.index)

# Class functions. Note how the parantheses invoke them. They are called "methods"
# print(df.drop())
print(df.describe())
print(df.head())


import numpy as np
import pandas as pd

def say_hello():
    print(np.array(["hello"]))
    return None

def random_phrase():
    adjectives = np.array(['blue', 'large', 'grainy', 
                        'substantial', 'potent', 'thermonuclear'])
    nouns = np.array(['food', 'house', 'tree', 'bicycle', 
                    'toupee', 'phone'])
    x = np.random.choice(adjectives)
    y = np.random.choice(nouns)
    return x + ' ' + y

def random_float(min_val, max_val):
    return round(np.random.uniform(low=min_val, high=max_val), 2)

def random_bowling_score():
    return np.random.randint(0, 300)

def silly_tuple():
    return (random_phrase(), random_float(1, 5), random_bowling_score())

def silly_tuple_list(num_tuples):
    return [silly_tuple() for i in range(num_tuples)]

def rem_outlier(df):
    for col in df.columns:
        upper_qr = np.percentile(df[col], 75)
        lower_qr = np.percentile(df[col], 25)
        iqr = (upper_qr - lower_qr) * 1.5
        upper_bound = upper_qr + iqr
        lower_bound = lower_qr - iqr
        print(upper_qr)
        print(upper_bound)
        for x in df.index:
            print(x)
            print(df.at[x, col])
            print(df[col])
            if (df.at[x, col] < lower_bound) or (df.at[x, col] > upper_bound):
                df.drop(x, inplace=True)
    return df



# TESTING
# print(random_phrase())
# print(random_float(3.0, 13.0))
# print(random_bowling_score())
# print(silly_tuple())
# print(silly_tuple_list(3))


# df = pd.DataFrame(
#     data={
#         'A':[9,3,6,3],
#         'B':[1,850,0,2],
#         'C':[2,5,8,1]
#     }
# )
# print(rem_outlier(df))


import numpy as np

class Student:
    books = 3

    def __init__(self, ID, major, in_PE=0):
        self.ID = ID
        self.major = major
        self.in_PE = in_PE

    def catch_phrase(self):
        print('Bet on it.')

    def join_PE(self):
        self.in_PE = 1

class BloomTechStudent(Student):

    def __init__(self, ID, major, track, in_PE=0):
        super().__init__(ID, major, in_PE)
        self.track = track
    
    def hack(self):
        print('You hacked Evil Corp!')

def student_generator(student_class=BloomTechStudent):
    student_list = []

    majors = np.array(['english','history','math','computer science'])
    tracks = np.array(['front-end','backend','data science','web3'])
    for i in range(30):
        new_student = student_class(np.random.randint(1, 1000),
                                    np.random.choice(majors),
                                    np.random.choice(tracks))
        student_list.append(new_student)
    return student_list

thirty_students = student_generator()
student_attributes = []
for student in thirty_students:
    student_attributes.append((student.ID, student.major, student.track))
print(student_attributes)


"""Creates vehicle classes"""
# Parent Class
class Vehicle:
    """Creates a vehicle class"""
    # static attribute
    wheels = 4

    # Constructor - defines user supplied attributes
    # 3 required attributes, 2 optional, 1 static (wheels)
    def __init__(self, make, model, color, year=2021, mileage=0):
        self.make= make
        self.model = model
        self.color = color
        self.year = year
        self.mileage = mileage

    def honk(self):
        """Prints honk"""
        print('HOOONK!')

    def drive(self, miles_driven):
        """Prints vroom"""
        print("VROOOM!")
        self.mileage = self.mileage + miles_driven


# Child Class, is more specific and inherits attributes and methods from the parent class
class Car(Vehicle):
    """Creates specific class of car"""
    def __init__(self, make, model, color, style, year=2021, mileage=0):
        # super() will inherit from the parent class
        super().__init__(make, model, color, year, mileage)

        #self.style is the only unique attribute
        self.style = style

# detect if the file is being imported in REPL or ran as a script

print('Running in a REPL or script')

# This condition will only be true if being ran as a script
if __name__ == '__main__':
    print("Running as a SCRIPT")
    


### IN style_example.py FILE###
"""Example of code review and linting"""
#what would you say if you were working with someone and this is the code they gave you?

import math
import sys

def example_1():
    ### THIS IS A LONG COMMENT AND should be
    ### wrapped to fit within a 72 character limit
    some_tuple =(1, 2, 3, 'a')
    some_variable={"long": """LONG CODE LINES should be wrapped within 
    79 character to prevent page cutoff stuff""",
                 'other':[math.pi, 100,200, 300, 9999292929292, """This IS a long string that
                  looks gross and goes beyond what it should"""],
                  "more": {"inner": "THIS whole logical line should be wrapped"},
                  "data": [444,5555,222,3,3,4,4,5,5,5,5,5,5,5]}
    return (some_tuple, some_variable)

def example_2():
    return {"has_key() is deprecated": True}

class Example3:
    def __init__(self, num):
        self.num = num
        self._square = (num+1)**2

    @property
    def square(self):
        return self._square

    def not_bar(self):
        some_string = """
                INDENTATION IN MULTIPLE STRINGS SHOULD NOT BE TOUCHED only actual code should be reindented,
    THIS IS MORE CODE
    """
        return (sys.path, some_string)
